#! /usr/bin/python

#   SuckNews 1.02 - a simple NNTP fetcher
#   Copyright (C) 2000 Robert Kerr <rkerr@go.to>

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#   SuckNews fetches news from an NNTP server, it can fetch whole articles or
#   just the header/body (specify -a, -b or -h on the commandline). A further
#   option is -x which will fetch the few headers available via XOVER and the
#   article body, this should be as fast as -b, ie quicker than -a. Fetched
#   news items are written to stdout, separated by .\n, status info is
#   written to stderr, so you can pipe stdout to a file or other program and
#   still get status info. SuckNews uses a file called groups in the current
#   directory to determine what groups to fetch and what the last fetched
#   article in each group was. The format of this file is simply -
#       <group name> <number of last fetched article>
#   eg.
#       comp.sys.acorn.programmer 39855
#   At the beginning of each fetch this file will be copied to groups.old
#   and as each group is fetched a fresh groups file, with an updated last
#   article count for each group will be written.

#   SuckNews has been tested on RISC OS and linux, it ought to work on any
#   system that has a python interpreter

import getopt, nntplib, os, string, sys

sys.stderr.write( "SuckNews 1.02, copyright (C) 2000 Robert Kerr <rkerr@go.to>\n" )
sys.stderr.write( "SuckNews is distributed in the hope that it will be useful, but with ABSOLUTELY\nNO WARRANTY, for more details see the GNU General Public License. This program\nis free software, you are welcome to redistribute it under the terms of the GPL.\n" ) 

if len( sys.argv ) < 2:
	raise "Syntax", sys.argv[0] + " [-h] [-b] [-x] [-a] <nntpserver>"
else:
	opts, arg = getopt.getopt( sys.argv[1:], "bhxa" )
	if len( arg ) == 1:
		server = arg[0]
		if len( opts ) == 0:
			opt = "-a"
		elif len( opts ) == 1:
			opt, val = opts[0]
		else:
			raise "Syntax", sys.argv[0] + " [-h] [-b] [-x] [-a] <nntpserver>"
	else:
		raise "Syntax", sys.argv[0] + " [-h] [-b] [-x] [-a] <nntpserver>"

max_size = 128

groupfile = open( "groups", "r" )
groups = groupfile.readlines()
groupfile.close()

os.remove( "groups.old" )
os.rename( "groups", "groups.old" )

groupdict = {}
idlist = []

for group in groups:
	name = string.split( group, " " )[0]
	last = string.split( group, " " )[1][:-1]
	groupdict[ name ] = last

fetcher = nntplib.NNTP( server )
sys.stderr.write( fetcher.getwelcome() + "\n" )

groupfile = open( "groups", "w" )

for group in groupdict.keys():
	resp, count, first, last, name = fetcher.group( group )
	sys.stderr.write( resp + "\n" )
	first = int(first)
	if first < int(groupdict[ group ]):
		first = int(groupdict[ group ])
	last = int(last)
	count = last - first
	if count == 0:
		sys.stderr.write( "No new articles in " + group + "\n" )
	else:
		sys.stderr.write( name + " has " + str(count) + " articles, range " + str(first) + " to " + str(last) + "\n" )
		# Some articles may have been cancelled, check what articles
		# are still available, also note first is actually the last
		# article we collected last time, so we don't want it
		sys.stderr.write( "Retrieving article information...\n" )
		resp, list = fetcher.xover( str(first + 1) , str(last) )
		articlelist = []
		if opt == "-x":
			headerdict = {}
		for item in list:
			article, subject, poster, date, id, references, size, lines = item
			# cull duplicates
			if id not in idlist:
				if int(size) < (max_size*1024):
					articlelist.append(article)
					idlist.append(id)
					if opt == "-x":
						headerdict[ article ] = "From: " + poster + "\nSubject: " + subject + "\nDate: " + date + "\nMessage-ID: " + id + "\n"
				else:
					sys.stderr.write( "Discarded article " + article + ", " + id + ", size greater than " + str(max_size) + "k\n")
					idlist.append(id)
			else:
				sys.stderr.write( "Culled duplicate of article " + id + "\n" )
		
		for article in articlelist:
			sys.stderr.write( "Fetching article " + article + " [ " + str(articlelist.index(article) + 1 ) + " of " + str(len(articlelist)) + " ]\n" )
			if opt == "-x":
				print headerdict[ article ]
			if opt == "-a" :
				resp, nr, id, list = fetcher.article( str(article) )
			elif opt in ["-b","-x"]:
				resp, nr, id, list = fetcher.body( str(article) )
			elif opt == "-h":
				resp, nr, id, list = fetcher.head( str(article) )
			print string.join( list, "\n" )
			print ".\n"
	groupfile.write( group + " " + str(last) + "\n")

sys.stderr.write( fetcher.quit() + "\n" )
groupfile.close()
